@@ -1,8 +1,22 @@
 setMethod("proj4string", signature(obj = "Spatial"),
-	function(obj) 
-		as.character(obj@proj4string@projargs)
+	function(obj) {
+                if (!is.null(comment(slot(obj, "proj4string"))))
+                    warning("CRS object has comment, which is lost in output")
+		res <- as.character(obj@proj4string@projargs)
+                res
+        }
 )
 
+setMethod("wkt", signature(obj = "Spatial"),
+	function(obj) {
+                comm <- comment(slot(obj, "proj4string"))
+                if (is.null(comm))
+                    warning("CRS object has no comment")
+		comm
+        }
+)
+
+
 ReplProj4string = function(obj, value) {
 	p4str <- value@projargs
 	ll <- FALSE
@@ -60,18 +74,28 @@ setReplaceMethod("proj4string", c("Spatial", "CRS"), ReplProj4string)
         attr(res, "details") <- list(W, E, S, N)
 	res
 }
+is.projectedSpatial <- function(obj) {
+	p4str <- slot(obj, "proj4string")
+	is.projected(p4str)
+}
 
-setMethod("is.projected", signature(obj = "Spatial"),
-	function(obj) {
-		p4str <- proj4string(obj)
-		if (is.na(p4str) || !nzchar(p4str)) 
-			return(as.logical(NA))
-		else {
-			res <- grep("longlat", p4str, fixed = TRUE)
-			if (length(res) == 0)
-				return(TRUE)
-			else
-				return(FALSE)
-		}
+setMethod("is.projected", signature(obj = "Spatial"), is.projectedSpatial)
+
+is.projectedCRS <- function(obj) {
+	p4str <- slot(obj, "projargs")
+	if (is.na(p4str) || !nzchar(p4str)) 
+		return(as.logical(NA))
+	else {
+		res <- grep("longlat", p4str, fixed = TRUE)
+		if (length(res) == 0)
+			return(TRUE)
+		else
+			return(FALSE)
 	}
-)
+}
+
+
+
+setMethod("is.projected", signature(obj = "CRS"), is.projectedCRS)
+
+
